/***************************************************************************
                          Default.h  -  description
                             -------------------
    begin                : Jul 2, 2019
    copyright            : (C) 2019 by Tomas Oberhuber et al.
    email                : tomas.oberhuber@fjfi.cvut.cz
 ***************************************************************************/

/* See Copyright Notice in tnl/Copyright */

// Implemented by: Jakub Klinkovsky

#pragma once

#include <TNL/Allocators/Host.h>
#include <TNL/Allocators/Cuda.h>
#include <TNL/Execution/Sequential.h>
#include <TNL/Execution/OpenMP.h>
#include <TNL/Execution/Cuda.h>

namespace TNL {
namespace Allocators {

/**
 * \brief A trait-like class used for the selection of a default allocators for
 *        given execution model.
 */
template< typename Execution >
struct Default;

//! Sets \ref Allocators::Host as the default allocator for \ref Execution::Sequential.
template<>
struct Default< Execution::Sequential >
{
   template< typename T >
   using Allocator = Allocators::Host< T >;
};

//! Sets \ref Allocators::Host as the default allocator for \ref Execution::OpenMP.
template<>
struct Default< Execution::OpenMP >
{
   template< typename T >
   using Allocator = Allocators::Host< T >;
};

//! Sets \ref Allocators::Cuda as the default allocator for \ref Execution::Cuda.
template<>
struct Default< Execution::Cuda >
{
   template< typename T >
   using Allocator = Allocators::Cuda< T >;
};

} // namespace Allocators
} // namespace TNL
