/***************************************************************************
                          StaticEllpackIndexMultimap.h  -  description
                             -------------------
    begin                : Sep 9, 2015
    copyright            : (C) 2015 by Tomas Oberhuber et al.
    email                : tomas.oberhuber@fjfi.cvut.cz
 ***************************************************************************/

/* See Copyright Notice in tnl/Copyright */

#pragma once

#include <TNL/Containers/Vector.h>
#include <TNL/Containers/Multimaps/StaticEllpackIndexMultimapValues.h>

namespace TNL {
namespace Containers {
namespace Multimaps {

template< typename Execution >
struct StaticEllpackIndexMultimapSliceSizeGetter
{
   static constexpr int SliceSize = 1;
};

template<>
struct StaticEllpackIndexMultimapSliceSizeGetter< Execution::Cuda >
{
   static constexpr int SliceSize = 32;
};

template< int ValuesCount,
          typename Index = int,
          typename Execution = Execution::OpenMP,
          typename LocalIndex = Index,
          int SliceSize = StaticEllpackIndexMultimapSliceSizeGetter< Execution >::SliceSize >
class StaticEllpackIndexMultimap
   : public Object
{
   public:
      using ExecutionType              = Execution;
      using IndexType                  = Index;
      using LocalIndexType             = LocalIndex;
      using ValuesAccessorType         = StaticEllpackIndexMultimapValues< ValuesCount, IndexType, ExecutionType, LocalIndexType, SliceSize >;
      using ConstValuesAccessorType    = StaticEllpackIndexMultimapValues< ValuesCount, const IndexType, ExecutionType, LocalIndexType, SliceSize >;

      StaticEllpackIndexMultimap() = default;

      template< typename Execution_ >
      StaticEllpackIndexMultimap( const StaticEllpackIndexMultimap< ValuesCount, Index, Execution_, LocalIndex, SliceSize >& other );

      template< typename Execution_ >
      StaticEllpackIndexMultimap& operator=( const StaticEllpackIndexMultimap< ValuesCount, Index, Execution_, LocalIndex, SliceSize >& other );

      void setKeysRange( const IndexType& keysRange );

      __cuda_callable__
      const IndexType getKeysRange() const;

      void allocate();

      template< typename Execution_ >
      void setLike( const StaticEllpackIndexMultimap< ValuesCount, Index, Execution_, LocalIndex, SliceSize >& other );

      __cuda_callable__
      ValuesAccessorType getValues( const IndexType& inputIndex );

      __cuda_callable__
      ConstValuesAccessorType getValues( const IndexType& inputIndex ) const;

      bool operator==( const StaticEllpackIndexMultimap& other ) const;

      void save( File& file ) const;

      void load( File& file );

      using Object::load;

      using Object::save;

      void print( std::ostream& str ) const;

   protected:
      Vector< IndexType, ExecutionType, IndexType > values;

      IndexType keysRange = 0;

      __cuda_callable__
      IndexType getAllocationKeysRange( IndexType keysRange ) const;

      // friend class is needed for templated assignment operators
      template< int ValuesCount_, typename Index_, typename Execution_, typename LocalIndex_, int SliceSize_ >
      friend class StaticEllpackIndexMultimap;
};

template< int ValuesCount,
          typename Index,
          typename Execution,
          typename LocalIndex,
          int SliceSize >
std::ostream& operator<<( std::ostream& str, const StaticEllpackIndexMultimap< ValuesCount, Index, Execution, LocalIndex, SliceSize >& multimap );

} // namespace Multimaps
} // namespace Containers
} // namespace TNL

#include <TNL/Containers/Multimaps/StaticEllpackIndexMultimap.hpp>
