/***************************************************************************
                          Grid1D_impl.h  -  description
                             -------------------
    begin                : Jan 16, 2013
    copyright            : (C) 2013 by Tomas Oberhuber
    email                : tomas.oberhuber@fjfi.cvut.cz
 ***************************************************************************/

/* See Copyright Notice in tnl/Copyright */

#pragma once

#include <fstream>
#include <iomanip>
#include <TNL/String.h>
#include <TNL/Assert.h>
#include <TNL/Meshes/GridDetails/GnuplotWriter.h>
#include <TNL/Meshes/GridDetails/GridEntityGetter_impl.h>
#include <TNL/Meshes/GridDetails/NeighborGridEntityGetter1D_impl.h>
#include <TNL/Meshes/GridDetails/Grid1D.h>
#include <TNL/Meshes/GridDetails/GridEntityMeasureGetter.h>

namespace TNL {
namespace Meshes {

template< typename Real,
          typename Execution,
          typename Index >
Grid< 1, Real, Execution, Index >::Grid()
: numberOfCells( 0 ),
  numberOfVertices( 0 ),
  distGrid(nullptr)
{
}

template< typename Real,
          typename Execution,
          typename Index >
Grid< 1, Real, Execution, Index >::Grid( const Index xSize )
: numberOfCells( 0 ),
  numberOfVertices( 0 ),
        distGrid(nullptr)
{
   this->setDimensions( xSize );
}

template< typename Real,
          typename Execution,
          typename Index >
String Grid< 1, Real, Execution, Index >::getSerializationType()
{
   return String( "Meshes::Grid< " ) +
          convertToString( getMeshDimension() ) + ", " +
          getType< RealType >() + ", " +
          getType< TNL::Execution::OpenMP >() + ", " +
          getType< IndexType >() + " >";
};

template< typename Real,
          typename Execution,
          typename Index >
String Grid< 1, Real, Execution, Index >::getSerializationTypeVirtual() const
{
   return this->getSerializationType();
};

template< typename Real,
          typename Execution,
          typename Index >
void Grid< 1, Real, Execution, Index >::computeSpaceSteps()
{
   if( this->getDimensions().x() != 0 )
   {
      this->spaceSteps.x() = this->proportions.x() / ( Real )  this->getDimensions().x();
      this->computeSpaceStepPowers();
   }
}

template< typename Real,
          typename Execution,
          typename Index >
void Grid< 1, Real, Execution, Index > ::computeSpaceStepPowers()
{
      const RealType& hx = this->spaceSteps.x();
      this->spaceStepsProducts[ 0 ] = 1.0 / ( hx * hx );
      this->spaceStepsProducts[ 1 ] = 1.0 / hx;
      this->spaceStepsProducts[ 2 ] = 1.0;
      this->spaceStepsProducts[ 3 ] = hx;
      this->spaceStepsProducts[ 4 ] = hx * hx;
   
}


template< typename Real,
          typename Execution,
          typename Index >
void Grid< 1, Real, Execution, Index > ::computeProportions()
{
    this->proportions.x()=this->dimensions.x()*this->spaceSteps.x();
}

template< typename Real,
          typename Execution,
          typename Index >
void Grid< 1, Real, Execution, Index > :: setOrigin( const PointType& origin)
{
   this->origin = origin;
}

template< typename Real,
          typename Execution,
          typename Index  >
void Grid< 1, Real, Execution, Index >::setDimensions( const Index xSize )
{
   TNL_ASSERT_GT( xSize, 0, "Grid size must be positive." );
   this->dimensions.x() = xSize;
   this->numberOfCells = xSize;
   this->numberOfVertices = xSize + 1;
   computeSpaceSteps();
}

template< typename Real,
          typename Execution,
          typename Index  >
void Grid< 1, Real, Execution, Index >::setDimensions( const CoordinatesType& dimensions )
{
   this->setDimensions( dimensions. x() );
}

template< typename Real,
          typename Execution,
          typename Index  >
__cuda_callable__ inline
const typename Grid< 1, Real, Execution, Index >::CoordinatesType&
   Grid< 1, Real, Execution, Index >::getDimensions() const
{
   return this->dimensions;
}

template< typename Real,
          typename Execution,
          typename Index >
void Grid< 1, Real, Execution, Index >::setDomain( const PointType& origin,
                                                     const PointType& proportions )
{
   this->origin = origin;
   this->proportions = proportions;
   computeSpaceSteps();
}

template< typename Real,
          typename Execution,
          typename Index  >
__cuda_callable__ inline
const typename Grid< 1, Real, Execution, Index >::PointType&
  Grid< 1, Real, Execution, Index >::getOrigin() const
{
   return this->origin;
}

template< typename Real,
          typename Execution,
          typename Index >
__cuda_callable__ inline
const typename Grid< 1, Real, Execution, Index >::PointType&
   Grid< 1, Real, Execution, Index >::getProportions() const
{
   return this->proportions;
}


template< typename Real,
          typename Execution,
          typename Index >
   template< int EntityDimension >
__cuda_callable__  inline
Index
Grid< 1, Real, Execution, Index >::
getEntitiesCount() const
{
   static_assert( EntityDimension <= 1 &&
                  EntityDimension >= 0, "Wrong grid entity dimensions." );

   switch( EntityDimension )
   {
      case 1:
         return this->numberOfCells;
      case 0:
         return this->numberOfVertices;
   }
   return -1;
}

template< typename Real,
          typename Execution,
          typename Index >
   template< typename Entity >
__cuda_callable__  inline
Index
Grid< 1, Real, Execution, Index >::
getEntitiesCount() const
{
   return getEntitiesCount< Entity::getEntityDimension() >();
}

template< typename Real,
          typename Execution,
          typename Index >
   template< typename Entity >
 __cuda_callable__ inline
Entity
Grid< 1, Real, Execution, Index >::
getEntity( const IndexType& entityIndex ) const
{
   static_assert( Entity::getEntityDimension() <= 1 &&
                  Entity::getEntityDimension() >= 0, "Wrong grid entity dimensions." );

   return GridEntityGetter< Grid, Entity >::getEntity( *this, entityIndex );
}

template< typename Real,
          typename Execution,
          typename Index >
   template< typename Entity >
__cuda_callable__ inline
Index
Grid< 1, Real, Execution, Index >::
getEntityIndex( const Entity& entity ) const
{
   static_assert( Entity::getEntityDimension() <= 1 &&
                  Entity::getEntityDimension() >= 0, "Wrong grid entity dimensions." );

   return GridEntityGetter< Grid, Entity >::getEntityIndex( *this, entity );
}

template< typename Real,
          typename Execution,
          typename Index >
__cuda_callable__ inline
const typename Grid< 1, Real, Execution, Index >::PointType&
Grid< 1, Real, Execution, Index >::
getSpaceSteps() const
{
   return this->spaceSteps;
}

template< typename Real,
          typename Execution,
          typename Index >
inline void 
Grid< 1, Real, Execution, Index >::
setSpaceSteps(const typename Grid< 1, Real, Execution, Index >::PointType& steps)
{
    this->spaceSteps=steps;
    computeSpaceStepPowers();
    computeProportions();
}

template< typename Real,
          typename Execution,
          typename Index >
   template< int xPow >
__cuda_callable__ inline
const Real&
Grid< 1, Real, Execution, Index >::
getSpaceStepsProducts() const
{
   static_assert( xPow >= -2 && xPow <= 2, "unsupported value of xPow" );
   return this->spaceStepsProducts[ xPow + 2 ];
}

template< typename Real,
          typename Execution,
          typename Index >
__cuda_callable__
const Real&
Grid< 1, Real, Execution, Index >::
getCellMeasure() const
{
   return this->template getSpaceStepsProducts< 1 >();
}

template< typename Real,
          typename Execution,
          typename Index >
__cuda_callable__ inline
Real Grid< 1, Real, Execution, Index >::
getSmallestSpaceStep() const
{
   return this->spaceSteps.x();
}

template< typename Real,
          typename Execution,
          typename Index >
   template< typename GridFunction >
typename GridFunction::RealType
Grid< 1, Real, Execution, Index >::getDifferenceAbsMax( const GridFunction& f1,
                                                        const GridFunction& f2 ) const
{
   typename GridFunction::RealType maxDiff( -1.0 );

   Cell cell( *this );
   for( cell.getCoordinates().x() = 0;
        cell.getCoordinates().x() < getDimensions().x();
        cell.getCoordinates().x()++ )
   {
      IndexType c = this->getEntityIndex( cell );
      maxDiff = max( maxDiff, abs( f1[ c ] - f2[ c ] ) );
   }
   return maxDiff;
}

template< typename Real,
          typename Execution,
          typename Index >
   template< typename GridFunction >
typename GridFunction::RealType
Grid< 1, Real, Execution, Index >::getDifferenceLpNorm( const GridFunction& f1,
                                                        const GridFunction& f2,
                                                        const typename GridFunction::RealType& p ) const
{
   typedef typename GridFunction::RealType FunctionRealType;
   FunctionRealType lpNorm( 0.0 ), cellVolume( this->getSpaceSteps().x() );

   Cell cell( *this );
   for( cell.getCoordinates().x() = 0;
        cell.getCoordinates().x() < getDimensions().x();
        cell.getCoordinates().x()++ )
   {
      IndexType c = this->getEntityIndex( cell );
      lpNorm += ::pow( abs( f1[ c ] - f2[ c ] ), p );
   }
   lpNorm *= cellVolume;
   return ::pow( lpNorm, 1.0 / p );
}

template< typename Real,
          typename Execution,
          typename Index >
void Grid< 1, Real, Execution, Index >:: setDistMesh(DistributedMeshType * distMesh)
{
    this->distGrid=distMesh;
}
   
template< typename Real,
          typename Execution,
          typename Index >
DistributedMeshes::DistributedMesh <Grid< 1, Real, Execution, Index >> * 
Grid< 1, Real, Execution, Index >:: getDistributedMesh(void) const
{
    return this->distGrid;
}
    
template< typename Real,
          typename Execution,
          typename Index >
void Grid< 1, Real, Execution, Index >::save( File& file ) const
{
   Object::save( file );
   this->origin.save( file );
   this->proportions.save( file );
   this->dimensions.save( file );
};

template< typename Real,
          typename Execution,
          typename Index >
void Grid< 1, Real, Execution, Index >::load( File& file )
{
   Object::load( file );
   CoordinatesType dimensions;
   this->origin.load( file );
   this->proportions.load( file );
   dimensions.load( file );
   this->setDimensions( dimensions );
};

template< typename Real,
          typename Execution,
          typename Index >
void Grid< 1, Real, Execution, Index >::save( const String& fileName ) const
{
   Object::save( fileName );
}

template< typename Real,
          typename Execution,
          typename Index >
void Grid< 1, Real, Execution, Index >::load( const String& fileName )
{
   Object::load( fileName );
}

template< typename Real,
          typename Execution,
          typename Index >
void
Grid< 1, Real, Execution, Index >::
writeProlog( Logger& logger ) const
{
   if( this->getDistributedMesh() && this->getDistributedMesh()->isDistributed() )
      return this->getDistributedMesh()->writeProlog( logger );
   logger.writeParameter( "Dimension:", getMeshDimension() );
   logger.writeParameter( "Domain origin:", this->origin );
   logger.writeParameter( "Domain proportions:", this->proportions );
   logger.writeParameter( "Domain dimensions:", this->dimensions );
   logger.writeParameter( "Space steps:", this->getSpaceSteps() );
   logger.writeParameter( "Number of cells:", getEntitiesCount< Cell >() );
   logger.writeParameter( "Number of vertices:", getEntitiesCount< Vertex >() );
}

} // namespace Meshes
} // namespace TNL
