/***************************************************************************
                          VectorTestSetup.h  -  description
                             -------------------
    begin                : Apr 30, 2019
    copyright            : (C) 2019 by Tomas Oberhuber
    email                : tomas.oberhuber@fjfi.cvut.cz
 ***************************************************************************/

/* See Copyright Notice in tnl/Copyright */

#pragma once

#ifdef HAVE_GTEST
#include <limits>

#include <TNL/Experimental/Arithmetics/Quad.h>
#include <TNL/Containers/Vector.h>
#include <TNL/Containers/VectorView.h>
#include "VectorHelperFunctions.h"

#include "gtest/gtest.h"

using namespace TNL;
using namespace TNL::Containers;
using namespace TNL::Arithmetics;

// test fixture for typed tests
template< typename Vector >
class VectorTest : public ::testing::Test
{
protected:
   using VectorType = Vector;
   using ViewType = VectorView< typename Vector::RealType, typename Vector::ExecutionType, typename Vector::IndexType >;
};

// types for which VectorTest is instantiated
// TODO: Quad must be fixed
using VectorTypes = ::testing::Types<
#ifndef HAVE_CUDA
   Vector< int,            Execution::OpenMP, short >,
   Vector< long,           Execution::OpenMP, short >,
   Vector< float,          Execution::OpenMP, short >,
   Vector< double,         Execution::OpenMP, short >,
   //Vector< Quad< float >,  Execution::OpenMP, short >,
   //Vector< Quad< double >, Execution::OpenMP, short >,
   Vector< int,            Execution::OpenMP, int >,
   Vector< long,           Execution::OpenMP, int >,
   Vector< float,          Execution::OpenMP, int >,
   Vector< double,         Execution::OpenMP, int >,
   //Vector< Quad< float >,  Execution::OpenMP, int >,
   //Vector< Quad< double >, Execution::OpenMP, int >,
   Vector< int,            Execution::OpenMP, long >,
   Vector< long,           Execution::OpenMP, long >,
   Vector< float,          Execution::OpenMP, long >,
   Vector< double,         Execution::OpenMP, long >
   //Vector< Quad< float >,  Execution::OpenMP, long >,
   //Vector< Quad< double >, Execution::OpenMP, long >
#endif
#ifdef HAVE_CUDA
   Vector< int,            Execution::Cuda, short >,
   Vector< long,           Execution::Cuda, short >,
   Vector< float,          Execution::Cuda, short >,
   Vector< double,         Execution::Cuda, short >,
   //Vector< Quad< float >,  Execution::Cuda, short >,
   //Vector< Quad< double >, Execution::Cuda, short >,
   Vector< int,            Execution::Cuda, int >,
   Vector< long,           Execution::Cuda, int >,
   Vector< float,          Execution::Cuda, int >,
   Vector< double,         Execution::Cuda, int >,
   //Vector< Quad< float >,  Execution::Cuda, int >,
   //Vector< Quad< double >, Execution::Cuda, int >,
   Vector< int,            Execution::Cuda, long >,
   Vector< long,           Execution::Cuda, long >,
   Vector< float,          Execution::Cuda, long >,
   Vector< double,         Execution::Cuda, long >
   //Vector< Quad< float >,  Execution::Cuda, long >,
   //Vector< Quad< double >, Execution::Cuda, long >
#endif
>;

TYPED_TEST_SUITE( VectorTest, VectorTypes );
#endif
